package nme.installer;


import nme.display.BitmapData;
import nme.media.Sound;
import nme.net.URLRequest;
import nme.text.Font;
import nme.utils.ByteArray;


/**
 * ...
 * @author Joshua Granick
 */

class Assets {

	
	public static var cachedBitmapData:Hash<BitmapData> = new Hash<BitmapData>();
	
	private static var initialized:Bool = false;
	private static var resourceNames:Hash <String> = new Hash <String> ();
	private static var resourceTypes:Hash <String> = new Hash <String> ();
	
	
	private static function initialize ():Void {
		
		if (!initialized) {
			
			resourceNames.set ("assets/block_32.png", "assets/block_32.png");
			resourceTypes.set ("assets/block_32.png", "image");
			resourceNames.set ("assets/block_red.png", "assets/block_red.png");
			resourceTypes.set ("assets/block_red.png", "image");
			resourceNames.set ("assets/block_solidBottom_32.png", "assets/block_solidBottom_32.png");
			resourceTypes.set ("assets/block_solidBottom_32.png", "image");
			resourceNames.set ("assets/block_transparent.png", "assets/block_transparent.png");
			resourceTypes.set ("assets/block_transparent.png", "image");
			resourceNames.set ("assets/block_white.png", "assets/block_white.png");
			resourceTypes.set ("assets/block_white.png", "image");
			resourceNames.set ("assets/block_white_transparent.png", "assets/block_white_transparent.png");
			resourceTypes.set ("assets/block_white_transparent.png", "image");
			resourceNames.set ("assets/floor_red.png", "assets/floor_red.png");
			resourceTypes.set ("assets/floor_red.png", "image");
			resourceNames.set ("assets/floor_red_32.png", "assets/floor_red_32.png");
			resourceTypes.set ("assets/floor_red_32.png", "image");
			resourceNames.set ("assets/floor_red_32x32.png", "assets/floor_red_32x32.png");
			resourceTypes.set ("assets/floor_red_32x32.png", "image");
			resourceNames.set ("assets/floor_shade_32.png", "assets/floor_shade_32.png");
			resourceTypes.set ("assets/floor_shade_32.png", "image");
			resourceNames.set ("assets/floor_shade_32x32.png", "assets/floor_shade_32x32.png");
			resourceTypes.set ("assets/floor_shade_32x32.png", "image");
			resourceNames.set ("assets/floor_transparent.png", "assets/floor_transparent.png");
			resourceTypes.set ("assets/floor_transparent.png", "image");
			resourceNames.set ("assets/floor_white.png", "assets/floor_white.png");
			resourceTypes.set ("assets/floor_white.png", "image");
			resourceNames.set ("assets/floor_white_32.png", "assets/floor_white_32.png");
			resourceTypes.set ("assets/floor_white_32.png", "image");
			resourceNames.set ("assets/floor_white_32x32.png", "assets/floor_white_32x32.png");
			resourceTypes.set ("assets/floor_white_32x32.png", "image");
			
			initialized = true;
			
		}
		
	}
	
	public static function getResourceName (id:String): String {
		initialize ();
      return resourceNames.get(id);
   }
	
	public static function getBitmapData (id:String, useCache:Bool = true):BitmapData {
		
		initialize ();
		
		if (resourceTypes.exists (id) && resourceTypes.get (id) == "image") {
			
			if (useCache && cachedBitmapData.exists (id)) {
				
				return cachedBitmapData.get (id);
				
			} else {
				
				var data = BitmapData.load (resourceNames.get (id));
				
				if (useCache) {
					
					cachedBitmapData.set (id, data);
					
				}
				
				return data;
				
			}
			
		} else {
			
			trace ("[nme.Assets] There is no BitmapData asset with an ID of \"" + id + "\"");
			
			return null;
			
		}
		
	}
	
	
	public static function getBytes (id:String):ByteArray {
		
		initialize ();
		
		if (resourceNames.exists (id)) {
			
			return ByteArray.readFile (resourceNames.get (id));
			
		} else {
			
			trace ("[nme.Assets] There is no String or ByteArray asset with an ID of \"" + id + "\"");
			
			return null;
			
		}
		
	}
	
	
	public static function getFont (id:String):Font {
		
		initialize ();
		
		if (resourceTypes.exists (id) && resourceTypes.get (id) == "font") {
			
			return new Font (resourceNames.get (id));
			
		} else {
			
			trace ("[nme.Assets] There is no Font asset with an ID of \"" + id + "\"");
			
			return null;
			
		}
		
	}
	
	
	public static function getSound (id:String):Sound {
		
		initialize ();
		
		if (resourceTypes.exists (id)) {
			
			if (resourceTypes.get (id) == "sound") {
				
				return new Sound (new URLRequest (resourceNames.get (id)), null, false);
				
			} else if (resourceTypes.get (id) == "music") {
				
				return new Sound (new URLRequest (resourceNames.get (id)), null, true);
				
			}
			
		}
		
		trace ("[nme.Assets] There is no Sound asset with an ID of \"" + id + "\"");
		
		return null;
		
	}
	
	
	public static function getText (id:String):String {
		
		var bytes = getBytes (id);
		
		if (bytes == null) {
			
			return null;
			
		} else {
			
			return bytes.readUTFBytes (bytes.length);
			
		}
		
	}
	
	
}
