package game.systems;
import game.components.CatComponent;
import game.components.GameComponent;
import game.enums.EnemyType;
import game.events.CreateCatEvent;
import game.events.CreateEnemyEvent;
import game.events.GameEvent;
import nme.geom.Point;
import nme.geom.Rectangle;
import se.salomonsson.ent.GameTime;
import se.salomonsson.ent.Sys;

/**
 * ...
 * @author Tommislav
 */

class GameSystem extends Sys
{
	private var _spawnKittenTimer:Int;
	private var _spawnTurretTimer:Int;
	private var _spawnSeekingTimer:Int;
	
	
	public function new() { super(); }
	
	override public function onAdded(sm, em):Void 
	{
		super.onAdded(sm, em);
		addListener(GameEvent.CAT_CREATED, onCatCreated);
	}
	override public function onRemoved():Void 
	{
		super.onRemoved();
		removeListener(GameEvent.CAT_CREATED, onCatCreated);
	}
	
	
	override public function tick(gt:GameTime):Void 
	{
		if (_spawnKittenTimer <= 0) {
			
			var numberOfKittens = em().getEWC([CatComponent]);
			if (numberOfKittens.length > 5) {
				_spawnKittenTimer += 20;
				return;
			}
			
			spawnNewKitten();
			_spawnKittenTimer = Std.int(100 + Math.random() * 400);
			
			if (--_spawnTurretTimer <= 0) {
				dispatch(new CreateEnemyEvent(CreateEnemyEvent.SPAWN, EnemyType.TURRET, -1, -1));
				_spawnTurretTimer = 4;
			}
			
			if (--_spawnSeekingTimer <= 0) {
				dispatch(new CreateEnemyEvent(CreateEnemyEvent.SPAWN, EnemyType.FIRE_TOWARDS_PLAYER, -1, -1));
				_spawnTurretTimer = 6;
			}
		}
		
		_spawnKittenTimer--;
		
	}
	
	private function spawnNewKitten() 
	{
		var levelBounds = em().getComp(GameComponent).levelBounds;
		var shrunkBounds = levelBounds.clone();
		shrunkBounds.x += 50;
		shrunkBounds.y += 50;
		shrunkBounds.width -= 100;
		shrunkBounds.height -= 100;
		
		var dirs = ["d", "l", "r", "u"];
		var opposite = ["u", "r", "l", "d"];
		var rnd = Std.int(Math.random() * 4);
		
		var start = getPointOnEdge(dirs[rnd], shrunkBounds);
		var end = getPointOnEdge(opposite[rnd], shrunkBounds);
		
		dispatch(new CreateCatEvent(CreateCatEvent.SPAWN, start.x, start.y, end.x, end.y));
	}
	
	private function getPointOnEdge(edge:String, bounds:Rectangle) {
		var p = new Point();
		if (edge == "u" || edge == "d") {
			p.x = Math.random() * bounds.width;
			p.y = (edge == "u") ? 0 : bounds.bottom;
		}
		
		if (edge == "l" || edge == "r") {
			p.y = Math.random() * bounds.height;
			p.x = (edge == "l") ? 0 : bounds.right;
		}
		return p;
	}
	
	
	
	private function onCatCreated(e:GameEvent):Void 
	{
		
		var kittenId:Int = e.data;
		
		var enemyTypes = [
			EnemyType.RANDOM,
			EnemyType.RANDOM,
			EnemyType.RANDOM,
			EnemyType.RANDOM,
			EnemyType.RANDOM,
			EnemyType.RANDOM,
			EnemyType.RANDOM,
			EnemyType.RANDOM_FAST,
			EnemyType.RANDOM_FAST,
			EnemyType.RANDOM_FAST,
			EnemyType.RANDOM_FAST,
			EnemyType.TURRET,
			EnemyType.TURRET,
			EnemyType.FIRE_TOWARDS_PLAYER,
			EnemyType.FIRE_TOWARDS_PLAYER,
			EnemyType.FIRE_TOWARDS_PLAYER,
		];
		
		var numShips = getNumberOfEscorts();
		var type = enemyTypes[Std.int(Math.random() * numShips * 2)];
		
		
		for (i in 0...numShips) {
			dispatch(new CreateEnemyEvent(CreateEnemyEvent.SPAWN, type, kittenId, i));
		}
	}
	
	private function getNumberOfEscorts() 
	{
		var gameComponent:GameComponent = em().getComp(GameComponent);
		var score = gameComponent.score;
		var numCats = gameComponent.catsTotal;
		
		var n = (score * 4) + numCats;
		
		if (n < 2)
			return 0;
		
		if (n < 60)
			return 1;
		
		if (n < 120)
			return 2;
			
		if (n < 240)
			return 4;
		
		if (n < 450)
			return 6;
			
		if (n < 800)
			return 7;
		
		
		return 8;
	}
}