package game.systems;
import game.components.DamagebleComponent;
import game.components.GameComponent;
import game.components.PlayerComponent;
import game.events.GameEvent;
import game.utils.ShadowTF;
import nme.display.Sprite;
import nme.Lib;
import nme.text.TextField;
import nme.text.TextFieldAutoSize;
import nme.text.TextFormat;
import se.salomonsson.ent.GameTime;
import se.salomonsson.ent.Sys;

/**
 * ...
 * @author Tommislav
 */

class RenderHudSystem extends Sys
{
	private var _money:ShadowTF;
	private var _health:ShadowTF;
	private var _score:ShadowTF;
	
	private var _lastHealth:Int;
	private var _lastMoney:Int;
	private var _lastScore:Int;
	
	public function new() 
	{
		super();
		_money = new ShadowTF();
		_health = new ShadowTF();
		_score = new ShadowTF();
		
		_health.x = 220;
		_money.x = 320;
		_score.y = 480 - 20;
		_score.x = 500;
		
		
		
		Lib.current.stage.addChild(_health);
		Lib.current.stage.addChild(_score);
		Lib.current.stage.addChild(_money);
	}
	
	override public function onAdded(sm, em):Void 
	{
		super.onAdded(sm, em);
		addListener(GameEvent.SHOP_ENTER, onShopEnter);
		addListener(GameEvent.FORCE_HUD_UPDATE, onForceHudUpdate);
		
	}
	
	override public function onRemoved():Void 
	{
		super.onRemoved();
		removeListener(GameEvent.SHOP_ENTER, onShopEnter);
		removeListener(GameEvent.FORCE_HUD_UPDATE, onForceHudUpdate);
	}
	
	
	private function onForceHudUpdate(e:GameEvent):Void 
	{
		tick(null);
	}
	
	
	private function onShopEnter(e:GameEvent):Void 
	{
		Lib.current.stage.addChild(_health);
		Lib.current.stage.addChild(_score);
		Lib.current.stage.addChild(_money);
	}
	
	
	override public function tick(gt:GameTime):Void 
	{
		var gameComponent = em().getComp(GameComponent);
		var player = em().getEWC([PlayerComponent]);
		var health = player[0].comp(DamagebleComponent).health;
		var money = gameComponent.money;
		var score = gameComponent.score;
		
		if (health != _lastHealth || money != _lastMoney || score != _lastScore) {
			_money.setText("MONEY: $ " + money);
			_health.setText("HEALTH: " + health);
			_score.setText("SCORE: " + score);
			
			_lastHealth = health;
			_lastMoney = money;
			_lastScore = score;
		}
		
	}
}