package game.systems;
import game.components.BitmapComponent;
import game.components.CenterPointPositionComponent;
import game.components.DamagebleComponent;
import game.components.GameComponent;
import game.components.GunComponent;
import game.components.PlayerComponent;
import game.components.ShopComponent;
import game.components.TileMapComponent;
import game.enums.TileId;
import game.events.GameEvent;
import game.spaceshop.ShopEvent;
import game.spaceshop.SpaceShopScreen;
import game.utils.TileData;
import game.utils.TileMap;
import nme.events.MouseEvent;
import nme.Lib;
import se.salomonsson.ent.GameTime;
import se.salomonsson.ent.Sys;
import se.salomonsson.game.components.CameraComponent;

/**
 * ...
 * @author Tommislav
 */

class UpdateSpaceShopSystem extends Sys
{
	private var _shoppingScreen:SpaceShopScreen;
	private var _alreadyBeenHit:Bool;
	private var _updateCurrentFrameCount:Int;
	private var _currentFrame:Int;
	
	private var _frames:Array<TileData>;
	
	public function new() 
	{
		super();
		_shoppingScreen = new SpaceShopScreen();
	}
	
	override public function onAdded(sm, em):Void 
	{
		super.onAdded(sm, em);
		var tileMap:TileMap = em.getComp(TileMapComponent).tileMap;
		_frames = [
			tileMap.getTile(TileId.SHOP_0),
			tileMap.getTile(TileId.SHOP_1)
		];
	}
	
	override public function tick(gt:GameTime):Void 
	{
		var shopList = em().getEWC([ShopComponent]);
		if (shopList.length > 0) {
			var shop = shopList[0];
			var pos = shop.comp(CenterPointPositionComponent);
			var camera = em().getComp(CameraComponent);
			if (camera.inView(pos)) {
				// update current frame
				if (++_updateCurrentFrameCount % 4 == 0) {
					var frameNum = ++_currentFrame % 2;
					var bmp = _frames[frameNum].getBitmap();
					shop.comp(BitmapComponent).newFrame(bmp.bitmapData);
				}
				
				
				
				// check for player collision
				
				var player = em().getEWC([PlayerComponent])[0];
				var playerPos = player.comp(CenterPointPositionComponent);
				var wantsToEnterShop = pos.intersects(playerPos);
				
				if (wantsToEnterShop && !_alreadyBeenHit) {
					_alreadyBeenHit = true;
					// ## LAUNCH SHOP!! ##
					
					var gameComp = em().getComp(GameComponent);
					var healthComp = player.comp(DamagebleComponent);
					var gunComp = player.comp(GunComponent);
					
					//gameComp.money = 200; // debug
					
					Lib.current.stage.addChild(_shoppingScreen);
					_shoppingScreen.openShop(gameComp, healthComp, gunComp);
					_shoppingScreen.addEventListener(ShopEvent.EXIT_SHOP, onExitShoppingScreen);
					_shoppingScreen.addEventListener(ShopEvent.BUY_ITEM, onItemBought);
					getManager().pause();
					
					dispatch(new GameEvent(GameEvent.SHOP_ENTER, null));
					
					if (gameComp.score > 50) {
						var levelBounds = gameComp.levelBounds;
						pos.x = (Math.random() * levelBounds.width - 256) + 128;
						pos.y = (Math.random() * levelBounds.height - 256) + 128;
					}
					
				}
				if (!wantsToEnterShop)
					_alreadyBeenHit = false;
			}
		}
	}
	
	private function onItemBought(e:ShopEvent):Void 
	{
		dispatch(new GameEvent(GameEvent.FORCE_HUD_UPDATE, null));
	}
	
	private function onExitShoppingScreen(e:ShopEvent):Void 
	{
		_shoppingScreen.removeEventListener(ShopEvent.EXIT_SHOP, onExitShoppingScreen);
		_shoppingScreen.removeEventListener(ShopEvent.BUY_ITEM, onItemBought);
		
		if (_shoppingScreen.parent != null) {
			_shoppingScreen.parent.removeChild(_shoppingScreen);
		}
		
		getManager().resume();
	}
	
}